@@ -1,22 +1,20 @@
-from __future__ import unicode_literals
-
-import re
 import tempfile
 import unittest
+from io import StringIO
 
 from django.contrib.gis import gdal
-from django.contrib.gis.db.models import Extent, MakeLine, Union
+from django.contrib.gis.db.models import Extent, MakeLine, Union, functions
 from django.contrib.gis.geos import (
     GeometryCollection, GEOSGeometry, LinearRing, LineString, MultiLineString,
     MultiPoint, MultiPolygon, Point, Polygon, fromstr,
 )
 from django.core.management import call_command
-from django.db import DatabaseError, connection
-from django.test import TestCase, ignore_warnings, skipUnlessDBFeature
-from django.utils import six
-from django.utils.deprecation import RemovedInDjango20Warning
+from django.db import DatabaseError, NotSupportedError, connection
+from django.test import TestCase, skipUnlessDBFeature
 
-from ..utils import oracle, postgis, skipUnlessGISLookup, spatialite
+from ..utils import (
+    mysql, no_oracle, oracle, postgis, skipUnlessGISLookup, spatialite,
+)
 from .models import (
     City, Country, Feature, MinusOneSRID, NonConcreteModel, PennsylvaniaCity,
     State, Track,
@@ -154,37 +152,39 @@ class GeoModelTest(TestCase):
         self.assertIsInstance(f_4.geom, GeometryCollection)
         self.assertEqual(f_3.geom, f_4.geom[2])
 
+    # TODO: fix on Oracle: ORA-22901: cannot compare nested table or VARRAY or
+    # LOB attributes of an object type.
+    @no_oracle
     @skipUnlessDBFeature("supports_transform")
     def test_inherited_geofields(self):
-        "Test GeoQuerySet methods on inherited Geometry fields."
+        "Database functions on inherited Geometry fields."
         # Creating a Pennsylvanian city.
         PennsylvaniaCity.objects.create(name='Mansfield', county='Tioga', point='POINT(-77.071445 41.823881)')
 
         # All transformation SQL will need to be performed on the
         # _parent_ table.
-        qs = PennsylvaniaCity.objects.transform(32128)
+        qs = PennsylvaniaCity.objects.annotate(new_point=functions.Transform('point', srid=32128))
 
         self.assertEqual(1, qs.count())
         for pc in qs:
-            self.assertEqual(32128, pc.point.srid)
+            self.assertEqual(32128, pc.new_point.srid)
 
     def test_raw_sql_query(self):
         "Testing raw SQL query."
         cities1 = City.objects.all()
-        # Only PostGIS would support a 'select *' query because of its recognized
-        # HEXEWKB format for geometry fields
-        as_text = 'ST_AsText(%s)' if postgis else connection.ops.select
-        cities2 = City.objects.raw(
-            'select id, name, %s from geoapp_city' % as_text % 'point'
-        )
-        self.assertEqual(len(cities1), len(list(cities2)))
-        self.assertIsInstance(cities2[0].point, Point)
+        point_select = connection.ops.select % 'point'
+        cities2 = list(City.objects.raw(
+            'select id, name, %s as point from geoapp_city' % point_select
+        ))
+        self.assertEqual(len(cities1), len(cities2))
+        with self.assertNumQueries(0):  # Ensure point isn't deferred.
+            self.assertIsInstance(cities2[0].point, Point)
 
     def test_dumpdata_loaddata_cycle(self):
         """
         Test a dumpdata/loaddata cycle with geographic data.
         """
-        out = six.StringIO()
+        out = StringIO()
         original_data = list(City.objects.all().order_by('name'))
         call_command('dumpdata', 'geoapp.City', stdout=out)
         result = out.getvalue()
@@ -196,7 +196,7 @@ class GeoModelTest(TestCase):
             tmp.write(result)
             tmp.seek(0)
             call_command('loaddata', tmp.name, verbosity=0)
-        self.assertListEqual(original_data, list(City.objects.all().order_by('name')))
+        self.assertEqual(original_data, list(City.objects.all().order_by('name')))
 
     @skipUnlessDBFeature("supports_empty_geometries")
     def test_empty_geometries(self):
@@ -301,9 +301,10 @@ class GeoLookupTest(TestCase):
         invalid_geom = fromstr('POLYGON((0 0, 0 1, 1 1, 1 0, 1 1, 1 0, 0 0))')
         State.objects.create(name='invalid', poly=invalid_geom)
         qs = State.objects.all()
-        if oracle:
+        if oracle or (mysql and connection.mysql_version < (8, 0, 0)):
             # Kansas has adjacent vertices with distance 6.99244813842e-12
             # which is smaller than the default Oracle tolerance.
+            # It's invalid on MySQL < 8 also.
             qs = qs.exclude(name='Kansas')
             self.assertEqual(State.objects.filter(name='Kansas', poly__isvalid=False).count(), 1)
         self.assertEqual(qs.filter(poly__isvalid=False).count(), 1)
@@ -385,6 +386,9 @@ class GeoLookupTest(TestCase):
         # Puerto Rico should be NULL (it's a commonwealth unincorporated territory)
         self.assertEqual(1, len(nullqs))
         self.assertEqual('Puerto Rico', nullqs[0].name)
+        # GeometryField=None is an alias for __isnull=True.
+        self.assertCountEqual(State.objects.filter(poly=None), nullqs)
+        self.assertCountEqual(State.objects.exclude(poly=None), validqs)
 
         # The valid states should be Colorado & Kansas
         self.assertEqual(2, len(validqs))
@@ -402,6 +406,29 @@ class GeoLookupTest(TestCase):
         State.objects.filter(name='Northern Mariana Islands').update(poly=None)
         self.assertIsNone(State.objects.get(name='Northern Mariana Islands').poly)
 
+    @skipUnlessDBFeature('supports_null_geometries', 'supports_crosses_lookup', 'supports_relate_lookup')
+    def test_null_geometries_excluded_in_lookups(self):
+        """NULL features are excluded in spatial lookup functions."""
+        null = State.objects.create(name='NULL', poly=None)
+        queries = [
+            ('equals', Point(1, 1)),
+            ('disjoint', Point(1, 1)),
+            ('touches', Point(1, 1)),
+            ('crosses', LineString((0, 0), (1, 1), (5, 5))),
+            ('within', Point(1, 1)),
+            ('overlaps', LineString((0, 0), (1, 1), (5, 5))),
+            ('contains', LineString((0, 0), (1, 1), (5, 5))),
+            ('intersects', LineString((0, 0), (1, 1), (5, 5))),
+            ('relate', (Point(1, 1), 'T*T***FF*')),
+            ('same_as', Point(1, 1)),
+            ('exact', Point(1, 1)),
+            ('coveredby', Point(1, 1)),
+            ('covers', Point(1, 1)),
+        ]
+        for lookup, geom in queries:
+            with self.subTest(lookup=lookup):
+                self.assertNotIn(null, State.objects.filter(**{'poly__%s' % lookup: geom}))
+
     @skipUnlessDBFeature("supports_relate_lookup")
     def test_relate_lookup(self):
         "Testing the 'relate' lookup type."
@@ -410,8 +437,8 @@ class GeoLookupTest(TestCase):
         pnt1 = fromstr('POINT (649287.0363174 4177429.4494686)', srid=2847)
         pnt2 = fromstr('POINT(-98.4919715741052 29.4333344025053)', srid=4326)
 
-        # Not passing in a geometry as first param should
-        # raise a type error when initializing the GeoQuerySet
+        # Not passing in a geometry as first param raises a TypeError when
+        # initializing the QuerySet.
         with self.assertRaises(ValueError):
             Country.objects.filter(mpoly__relate=(23, 'foo'))
 
@@ -447,67 +474,23 @@ class GeoLookupTest(TestCase):
             self.assertEqual('Texas', Country.objects.get(mpoly__relate=(pnt2, intersects_mask)).name)
             self.assertEqual('Lawrence', City.objects.get(point__relate=(ks.poly, intersects_mask)).name)
 
+        # With a complex geometry expression
+        mask = 'anyinteract' if oracle else within_mask
+        self.assertFalse(City.objects.exclude(point__relate=(functions.Union('point', 'point'), mask)))
+
+    def test_gis_lookups_with_complex_expressions(self):
+        multiple_arg_lookups = {'dwithin', 'relate'}  # These lookups are tested elsewhere.
+        lookups = connection.ops.gis_operators.keys() - multiple_arg_lookups
+        self.assertTrue(lookups, 'No lookups found')
+        for lookup in lookups:
+            with self.subTest(lookup):
+                City.objects.filter(**{'point__' + lookup: functions.Union('point', 'point')}).exists()
+
 
-@ignore_warnings(category=RemovedInDjango20Warning)
 class GeoQuerySetTest(TestCase):
+    # TODO: GeoQuerySet is removed, organize these test better.
     fixtures = ['initial']
 
-    # Please keep the tests in GeoQuerySet method's alphabetic order
-
-    @skipUnlessDBFeature("has_centroid_method")
-    def test_centroid(self):
-        "Testing the `centroid` GeoQuerySet method."
-        qs = State.objects.exclude(poly__isnull=True).centroid()
-        if oracle:
-            tol = 0.1
-        elif spatialite:
-            tol = 0.000001
-        else:
-            tol = 0.000000001
-        for s in qs:
-            self.assertTrue(s.poly.centroid.equals_exact(s.centroid, tol))
-
-    @skipUnlessDBFeature(
-        "has_difference_method", "has_intersection_method",
-        "has_sym_difference_method", "has_union_method")
-    def test_diff_intersection_union(self):
-        "Testing the `difference`, `intersection`, `sym_difference`, and `union` GeoQuerySet methods."
-        geom = Point(5, 23)
-        qs = Country.objects.all().difference(geom).sym_difference(geom).union(geom)
-
-        # XXX For some reason SpatiaLite does something screwy with the Texas geometry here.  Also,
-        # XXX it doesn't like the null intersection.
-        if spatialite:
-            qs = qs.exclude(name='Texas')
-        else:
-            qs = qs.intersection(geom)
-
-        for c in qs:
-            if oracle:
-                # Should be able to execute the queries; however, they won't be the same
-                # as GEOS (because Oracle doesn't use GEOS internally like PostGIS or
-                # SpatiaLite).
-                pass
-            else:
-                if spatialite:
-                    # Spatialite `difference` doesn't have an SRID
-                    self.assertEqual(c.mpoly.difference(geom).wkt, c.difference.wkt)
-                else:
-                    self.assertEqual(c.mpoly.difference(geom), c.difference)
-                    self.assertEqual(c.mpoly.intersection(geom), c.intersection)
-                # Ordering might differ in collections
-                self.assertSetEqual(set(g.wkt for g in c.mpoly.sym_difference(geom)),
-                                    set(g.wkt for g in c.sym_difference))
-                self.assertSetEqual(set(g.wkt for g in c.mpoly.union(geom)),
-                                    set(g.wkt for g in c.union))
-
-    @skipUnlessDBFeature("has_envelope_method")
-    def test_envelope(self):
-        "Testing the `envelope` GeoQuerySet method."
-        countries = Country.objects.all().envelope()
-        for country in countries:
-            self.assertIsInstance(country.envelope, Polygon)
-
     @skipUnlessDBFeature("supports_extent_aggr")
     def test_extent(self):
         """
@@ -533,138 +516,12 @@ class GeoQuerySetTest(TestCase):
         extent2 = City.objects.all()[:3].aggregate(Extent('point'))['point__extent']
         self.assertNotEqual(extent1, extent2)
 
-    @skipUnlessDBFeature("has_force_rhr_method")
-    def test_force_rhr(self):
-        "Testing GeoQuerySet.force_rhr()."
-        rings = (
-            ((0, 0), (5, 0), (0, 5), (0, 0)),
-            ((1, 1), (1, 3), (3, 1), (1, 1)),
-        )
-        rhr_rings = (
-            ((0, 0), (0, 5), (5, 0), (0, 0)),
-            ((1, 1), (3, 1), (1, 3), (1, 1)),
-        )
-        State.objects.create(name='Foo', poly=Polygon(*rings))
-        s = State.objects.force_rhr().get(name='Foo')
-        self.assertEqual(rhr_rings, s.force_rhr.coords)
-
-    @skipUnlessDBFeature("has_geohash_method")
-    def test_geohash(self):
-        "Testing GeoQuerySet.geohash()."
-        # Reference query:
-        # SELECT ST_GeoHash(point) FROM geoapp_city WHERE name='Houston';
-        # SELECT ST_GeoHash(point, 5) FROM geoapp_city WHERE name='Houston';
-        ref_hash = '9vk1mfq8jx0c8e0386z6'
-        h1 = City.objects.geohash().get(name='Houston')
-        h2 = City.objects.geohash(precision=5).get(name='Houston')
-        self.assertEqual(ref_hash, h1.geohash)
-        self.assertEqual(ref_hash[:5], h2.geohash)
-
-    def test_geojson(self):
-        "Testing GeoJSON output from the database using GeoQuerySet.geojson()."
-        # Only PostGIS and SpatiaLite support GeoJSON.
-        if not connection.ops.geojson:
-            with self.assertRaises(NotImplementedError):
-                Country.objects.all().geojson(field_name='mpoly')
-            return
-
-        pueblo_json = '{"type":"Point","coordinates":[-104.609252,38.255001]}'
-        houston_json = (
-            '{"type":"Point","crs":{"type":"name","properties":'
-            '{"name":"EPSG:4326"}},"coordinates":[-95.363151,29.763374]}'
-        )
-        victoria_json = (
-            '{"type":"Point","bbox":[-123.30519600,48.46261100,-123.30519600,48.46261100],'
-            '"coordinates":[-123.305196,48.462611]}'
-        )
-        chicago_json = (
-            '{"type":"Point","crs":{"type":"name","properties":{"name":"EPSG:4326"}},'
-            '"bbox":[-87.65018,41.85039,-87.65018,41.85039],"coordinates":[-87.65018,41.85039]}'
-        )
-        if spatialite:
-            victoria_json = (
-                '{"type":"Point","bbox":[-123.305196,48.462611,-123.305196,48.462611],'
-                '"coordinates":[-123.305196,48.462611]}'
-            )
-
-        # Precision argument should only be an integer
-        with self.assertRaises(TypeError):
-            City.objects.geojson(precision='foo')
-
-        # Reference queries and values.
-        # SELECT ST_AsGeoJson("geoapp_city"."point", 8, 0)
-        # FROM "geoapp_city" WHERE "geoapp_city"."name" = 'Pueblo';
-        self.assertEqual(pueblo_json, City.objects.geojson().get(name='Pueblo').geojson)
-
-        # SELECT ST_AsGeoJson("geoapp_city"."point", 8, 2) FROM "geoapp_city"
-        # WHERE "geoapp_city"."name" = 'Houston';
-        # This time we want to include the CRS by using the `crs` keyword.
-        self.assertEqual(houston_json, City.objects.geojson(crs=True, model_att='json').get(name='Houston').json)
-
-        # SELECT ST_AsGeoJson("geoapp_city"."point", 8, 1) FROM "geoapp_city"
-        # WHERE "geoapp_city"."name" = 'Houston';
-        # This time we include the bounding box by using the `bbox` keyword.
-        self.assertEqual(victoria_json, City.objects.geojson(bbox=True).get(name='Victoria').geojson)
-
-        # SELECT ST_AsGeoJson("geoapp_city"."point", 5, 3) FROM "geoapp_city"
-        # WHERE "geoapp_city"."name" = 'Chicago';
-        # Finally, we set every available keyword.
-        self.assertEqual(
-            chicago_json,
-            City.objects.geojson(bbox=True, crs=True, precision=5).get(name='Chicago').geojson
-        )
-
-    @skipUnlessDBFeature("has_gml_method")
-    def test_gml(self):
-        "Testing GML output from the database using GeoQuerySet.gml()."
-        # Should throw a TypeError when trying to obtain GML from a
-        # non-geometry field.
-        qs = City.objects.all()
-        with self.assertRaises(TypeError):
-            qs.gml(field_name='name')
-        ptown1 = City.objects.gml(field_name='point', precision=9).get(name='Pueblo')
-        ptown2 = City.objects.gml(precision=9).get(name='Pueblo')
-
-        if oracle:
-            # No precision parameter for Oracle :-/
-            gml_regex = re.compile(
-                r'^<gml:Point srsName="EPSG:4326" xmlns:gml="http://www.opengis.net/gml">'
-                r'<gml:coordinates decimal="\." cs="," ts=" ">-104.60925\d+,38.25500\d+ '
-                r'</gml:coordinates></gml:Point>'
-            )
-        else:
-            gml_regex = re.compile(
-                r'^<gml:Point srsName="EPSG:4326"><gml:coordinates>'
-                r'-104\.60925\d+,38\.255001</gml:coordinates></gml:Point>'
-            )
-
-        for ptown in [ptown1, ptown2]:
-            self.assertTrue(gml_regex.match(ptown.gml))
-
-        if postgis:
-            self.assertIn('<gml:pos srsDimension="2">', City.objects.gml(version=3).get(name='Pueblo').gml)
-
-    @skipUnlessDBFeature("has_kml_method")
-    def test_kml(self):
-        "Testing KML output from the database using GeoQuerySet.kml()."
-        # Should throw a TypeError when trying to obtain KML from a
-        #  non-geometry field.
-        qs = City.objects.all()
-        with self.assertRaises(TypeError):
-            qs.kml('name')
-
-        # Ensuring the KML is as expected.
-        ptown1 = City.objects.kml(field_name='point', precision=9).get(name='Pueblo')
-        ptown2 = City.objects.kml(precision=9).get(name='Pueblo')
-        for ptown in [ptown1, ptown2]:
-            self.assertEqual('<Point><coordinates>-104.609252,38.255001</coordinates></Point>', ptown.kml)
-
     def test_make_line(self):
         """
         Testing the `MakeLine` aggregate.
         """
         if not connection.features.supports_make_line_aggr:
-            with self.assertRaises(NotImplementedError):
+            with self.assertRaises(NotSupportedError):
                 City.objects.all().aggregate(MakeLine('point'))
             return
 
@@ -686,181 +543,6 @@ class GeoQuerySetTest(TestCase):
             "%s != %s" % (ref_line, line)
         )
 
-    @skipUnlessDBFeature("has_num_geom_method")
-    def test_num_geom(self):
-        "Testing the `num_geom` GeoQuerySet method."
-        # Both 'countries' only have two geometries.
-        for c in Country.objects.num_geom():
-            self.assertEqual(2, c.num_geom)
-
-        for c in City.objects.filter(point__isnull=False).num_geom():
-            # Oracle and PostGIS 2.0+ will return 1 for the number of
-            # geometries on non-collections.
-            self.assertEqual(1, c.num_geom)
-
-    @skipUnlessDBFeature("supports_num_points_poly")
-    def test_num_points(self):
-        "Testing the `num_points` GeoQuerySet method."
-        for c in Country.objects.num_points():
-            self.assertEqual(c.mpoly.num_points, c.num_points)
-
-        if not oracle:
-            # Oracle cannot count vertices in Point geometries.
-            for c in City.objects.num_points():
-                self.assertEqual(1, c.num_points)
-
-    @skipUnlessDBFeature("has_point_on_surface_method")
-    def test_point_on_surface(self):
-        "Testing the `point_on_surface` GeoQuerySet method."
-        # Reference values.
-        if oracle:
-            # SELECT SDO_UTIL.TO_WKTGEOMETRY(SDO_GEOM.SDO_POINTONSURFACE(GEOAPP_COUNTRY.MPOLY, 0.05))
-            # FROM GEOAPP_COUNTRY;
-            ref = {'New Zealand': fromstr('POINT (174.616364 -36.100861)', srid=4326),
-                   'Texas': fromstr('POINT (-103.002434 36.500397)', srid=4326),
-                   }
-
-        else:
-            # Using GEOSGeometry to compute the reference point on surface values
-            # -- since PostGIS also uses GEOS these should be the same.
-            ref = {'New Zealand': Country.objects.get(name='New Zealand').mpoly.point_on_surface,
-                   'Texas': Country.objects.get(name='Texas').mpoly.point_on_surface
-                   }
-
-        for c in Country.objects.point_on_surface():
-            if spatialite:
-                # XXX This seems to be a WKT-translation-related precision issue?
-                tol = 0.00001
-            else:
-                tol = 0.000000001
-            self.assertTrue(ref[c.name].equals_exact(c.point_on_surface, tol))
-
-    @skipUnlessDBFeature("has_reverse_method")
-    def test_reverse_geom(self):
-        "Testing GeoQuerySet.reverse_geom()."
-        coords = [(-95.363151, 29.763374), (-95.448601, 29.713803)]
-        Track.objects.create(name='Foo', line=LineString(coords))
-        t = Track.objects.reverse_geom().get(name='Foo')
-        coords.reverse()
-        self.assertEqual(tuple(coords), t.reverse_geom.coords)
-        if oracle:
-            with self.assertRaises(TypeError):
-                State.objects.reverse_geom()
-
-    @skipUnlessDBFeature("has_scale_method")
-    def test_scale(self):
-        "Testing the `scale` GeoQuerySet method."
-        xfac, yfac = 2, 3
-        tol = 5  # XXX The low precision tolerance is for SpatiaLite
-        qs = Country.objects.scale(xfac, yfac, model_att='scaled')
-        for c in qs:
-            for p1, p2 in zip(c.mpoly, c.scaled):
-                for r1, r2 in zip(p1, p2):
-                    for c1, c2 in zip(r1.coords, r2.coords):
-                        self.assertAlmostEqual(c1[0] * xfac, c2[0], tol)
-                        self.assertAlmostEqual(c1[1] * yfac, c2[1], tol)
-
-    @skipUnlessDBFeature("has_snap_to_grid_method")
-    def test_snap_to_grid(self):
-        "Testing GeoQuerySet.snap_to_grid()."
-        # Let's try and break snap_to_grid() with bad combinations of arguments.
-        for bad_args in ((), range(3), range(5)):
-            with self.assertRaises(ValueError):
-                Country.objects.snap_to_grid(*bad_args)
-        for bad_args in (('1.0',), (1.0, None), tuple(map(six.text_type, range(4)))):
-            with self.assertRaises(TypeError):
-                Country.objects.snap_to_grid(*bad_args)
-
-        # Boundary for San Marino, courtesy of Bjorn Sandvik of thematicmapping.org
-        # from the world borders dataset he provides.
-        wkt = ('MULTIPOLYGON(((12.41580 43.95795,12.45055 43.97972,12.45389 43.98167,'
-               '12.46250 43.98472,12.47167 43.98694,12.49278 43.98917,'
-               '12.50555 43.98861,12.51000 43.98694,12.51028 43.98277,'
-               '12.51167 43.94333,12.51056 43.93916,12.49639 43.92333,'
-               '12.49500 43.91472,12.48778 43.90583,12.47444 43.89722,'
-               '12.46472 43.89555,12.45917 43.89611,12.41639 43.90472,'
-               '12.41222 43.90610,12.40782 43.91366,12.40389 43.92667,'
-               '12.40500 43.94833,12.40889 43.95499,12.41580 43.95795)))')
-        Country.objects.create(name='San Marino', mpoly=fromstr(wkt))
-
-        # Because floating-point arithmetic isn't exact, we set a tolerance
-        # to pass into GEOS `equals_exact`.
-        tol = 0.000000001
-
-        # SELECT AsText(ST_SnapToGrid("geoapp_country"."mpoly", 0.1)) FROM "geoapp_country"
-        # WHERE "geoapp_country"."name" = 'San Marino';
-        ref = fromstr('MULTIPOLYGON(((12.4 44,12.5 44,12.5 43.9,12.4 43.9,12.4 44)))')
-        self.assertTrue(ref.equals_exact(Country.objects.snap_to_grid(0.1).get(name='San Marino').snap_to_grid, tol))
-
-        # SELECT AsText(ST_SnapToGrid("geoapp_country"."mpoly", 0.05, 0.23)) FROM "geoapp_country"
-        # WHERE "geoapp_country"."name" = 'San Marino';
-        ref = fromstr('MULTIPOLYGON(((12.4 43.93,12.45 43.93,12.5 43.93,12.45 43.93,12.4 43.93)))')
-        self.assertTrue(
-            ref.equals_exact(Country.objects.snap_to_grid(0.05, 0.23).get(name='San Marino').snap_to_grid, tol)
-        )
-
-        # SELECT AsText(ST_SnapToGrid("geoapp_country"."mpoly", 0.5, 0.17, 0.05, 0.23)) FROM "geoapp_country"
-        # WHERE "geoapp_country"."name" = 'San Marino';
-        ref = fromstr(
-            'MULTIPOLYGON(((12.4 43.87,12.45 43.87,12.45 44.1,12.5 44.1,12.5 43.87,12.45 43.87,12.4 43.87)))'
-        )
-        self.assertTrue(
-            ref.equals_exact(
-                Country.objects.snap_to_grid(0.05, 0.23, 0.5, 0.17).get(name='San Marino').snap_to_grid,
-                tol
-            )
-        )
-
-    @skipUnlessDBFeature("has_svg_method")
-    def test_svg(self):
-        "Testing SVG output using GeoQuerySet.svg()."
-
-        with self.assertRaises(TypeError):
-            City.objects.svg(precision='foo')
-        # SELECT AsSVG(geoapp_city.point, 0, 8) FROM geoapp_city WHERE name = 'Pueblo';
-        svg1 = 'cx="-104.609252" cy="-38.255001"'
-        # Even though relative, only one point so it's practically the same except for
-        # the 'c' letter prefix on the x,y values.
-        svg2 = svg1.replace('c', '')
-        self.assertEqual(svg1, City.objects.svg().get(name='Pueblo').svg)
-        self.assertEqual(svg2, City.objects.svg(relative=5).get(name='Pueblo').svg)
-
-    @skipUnlessDBFeature("has_transform_method")
-    def test_transform(self):
-        "Testing the transform() GeoQuerySet method."
-        # Pre-transformed points for Houston and Pueblo.
-        htown = fromstr('POINT(1947516.83115183 6322297.06040572)', srid=3084)
-        ptown = fromstr('POINT(992363.390841912 481455.395105533)', srid=2774)
-        prec = 3  # Precision is low due to version variations in PROJ and GDAL.
-
-        # Asserting the result of the transform operation with the values in
-        #  the pre-transformed points.  Oracle does not have the 3084 SRID.
-        if not oracle:
-            h = City.objects.transform(htown.srid).get(name='Houston')
-            self.assertEqual(3084, h.point.srid)
-            self.assertAlmostEqual(htown.x, h.point.x, prec)
-            self.assertAlmostEqual(htown.y, h.point.y, prec)
-
-        p1 = City.objects.transform(ptown.srid, field_name='point').get(name='Pueblo')
-        p2 = City.objects.transform(srid=ptown.srid).get(name='Pueblo')
-        for p in [p1, p2]:
-            self.assertEqual(2774, p.point.srid)
-            self.assertAlmostEqual(ptown.x, p.point.x, prec)
-            self.assertAlmostEqual(ptown.y, p.point.y, prec)
-
-    @skipUnlessDBFeature("has_translate_method")
-    def test_translate(self):
-        "Testing the `translate` GeoQuerySet method."
-        xfac, yfac = 5, -23
-        qs = Country.objects.translate(xfac, yfac, model_att='translated')
-        for c in qs:
-            for p1, p2 in zip(c.mpoly, c.translated):
-                for r1, r2 in zip(p1, p2):
-                    for c1, c2 in zip(r1.coords, r2.coords):
-                        # XXX The low precision is for SpatiaLite
-                        self.assertAlmostEqual(c1[0] + xfac, c2[0], 5)
-                        self.assertAlmostEqual(c1[1] + yfac, c2[1], 5)
-
     @skipUnlessDBFeature('supports_union_aggr')
     def test_unionagg(self):
         """
